"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const make_temp_dir_1 = require("./make_temp_dir");
const output_1 = require("./output");
const readable_dir_1 = require("./readable_dir");
const temp_dir_1 = require("./temp_dir");
/**
 * Create readable dir.
 * @param dir
 */
function createReadableDir(dir) {
    return new readable_dir_1.default(dir);
}
exports.createReadableDir = createReadableDir;
/**
 * Create temporary dir.
 */
function createTempDir() {
    return make_temp_dir_1.default().then((dir) => {
        return new temp_dir_1.default(dir);
    });
}
exports.createTempDir = createTempDir;
/**
 * Create a build output helper with the specified builder.
 * @param builder a broccoli builder.
 */
function wrapBuilder(builder) {
    return new output_1.default(builder);
}
exports.wrapBuilder = wrapBuilder;
/**
 * Create a build output helper with the specified output node using
 * the broccoli `Builder`.
 * @param outputNode
 */
function createBuilder(outputNode) {
    const Builder = require("broccoli").Builder;
    return wrapBuilder(new Builder(outputNode));
}
exports.createBuilder = createBuilder;
/**
 * Create a build output helper with the specified output node and a
 * promise for the initial build.
 * @param outputNode
 */
function buildOutput(outputNode) {
    let output = createBuilder(outputNode);
    return output.build().then(() => output);
}
exports.buildOutput = buildOutput;
//# sourceMappingURL=index.js.map